import { animate, AUTO_STYLE, state, style, transition, trigger } from '@angular/animations';
import { Component, OnInit } from '@angular/core';
import { UntypedFormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import * as lodash from 'lodash';
import { DateTime } from 'luxon';
import { structureFormStep } from '../../form/form-view/structure-form/structureFormStep.enum';
import { Structure } from '../../models/structure.model';
import { NotificationService } from '../../services/notification.service';
import { StructureService } from '../../services/structure.service';
import { ButtonType } from '../../shared/components/button/buttonType.enum';
import { Demarches } from '../../shared/enum/demarches.enum';
import { FreeWorkshop } from '../../shared/enum/freeWorkshop.enum';
import { OtherServices } from '../../shared/enum/otherServices.enum';
import { AccessModality } from '../../structure-list/enum/access-modality.enum';
import { Equipment } from '../../structure-list/enum/equipment.enum';
import { PublicCategorie } from '../../structure-list/enum/public.enum';
import { Category } from '../../structure-list/models/category.model';
import { Module } from '../../structure-list/models/module.model';
import { SearchService } from '../../structure-list/services/search.service';
import { formUtils, IStructureSummary } from '../../utils/formUtils';
import { Utils } from '../../utils/utils';

@Component({
  selector: 'app-structure-edition-summary',
  templateUrl: './structure-edition-summary.component.html',
  styleUrls: ['./structure-edition-summary.component.scss'],
  animations: [
    trigger('show', [
      state('true', style({ height: AUTO_STYLE, visibility: AUTO_STYLE, margin: '8px 0' })),
      state('false', style({ height: '0px', visibility: 'hidden', margin: '0' })),
      transition('true => false', animate('300ms ease-out')),
      transition('false => true', animate('300ms ease-out')),
    ]),
  ],
})
export class StructureEditionSummaryComponent implements OnInit {
  public structure: Structure;
  public structureForm: UntypedFormGroup;
  public summary: IStructureSummary[] = new formUtils().structureSummary;
  public structureFormStep = structureFormStep;
  public buttonTypeEnum = ButtonType;
  public equipmentEnum = Equipment;

  //Digital learning
  public baseSkillssReferentiel: Category;
  public accessRightsReferentiel: Category;
  public digitalCultureSecuritysReferentiel: Category;
  public socialAndProfessionalsReferentiel: Category;
  public parentingHelpsReferentiel: Category;
  public baseSkills: Module[];
  public accessRights: Module[];
  public parentingHelp: Module[];
  public socialAndProfessional: Module[];
  public digitalCultureSecurity: Module[];
  public showBaseSkills = false;
  public showAccessRights = false;
  public showParentingHelp = false;
  public showSocialAndProfessional = false;
  public showDigitalSecurity = false;

  public mailHref: string;
  // set to true when last updated date of a structure is > 6 months
  public isUpdateStructure = false;

  constructor(
    private notificationService: NotificationService,
    private route: ActivatedRoute,
    private searchService: SearchService,
    private structureService: StructureService,
    private utils: Utils,
    public router: Router
  ) {}

  // There is a non-blocking error in console when open edit mode that occurs only in development mode, please refer https://angular.io/errors/NG0100 for more info

  ngOnInit(): void {
    this.route.data.subscribe((data) => {
      if (data.structure) {
        this.structure = new Structure(data.structure);
        this.structureForm = new formUtils().createStructureForm(this.structure, true);
        if (this.structure.contactMail) this.mailHref = `mailto:${this.structure.contactMail}`;

        const updatedAt = DateTime.fromISO(this.structure.updatedAt);
        const sixMonthsAgo = DateTime.local().minus({ month: 6 });
        if (updatedAt < sixMonthsAgo) this.isUpdateStructure = true;
      }
      this.initForm();
    });
  }

  public goBack(): void {
    history.back();
  }
  public goToEdit(step: structureFormStep): void {
    this.router.navigate(['/form/structure', this.structure._id, structureFormStep[step]]);
  }

  public isFormValid(): boolean {
    for (const field in this.structure) {
      if (!['_id', 'numero', 'createdAt', 'updatedAt'].includes(field)) {
        if (this.structureForm.get(field)?.valid === false) return false;
      }
    }
    return true;
  }

  public updateStructure(): void {
    this.structureService.editStructure({}, this.structure._id).subscribe(() => {
      this.notificationService.showSuccess('La structure a bien été mise à jour.', '');
      this.isUpdateStructure = false;
    });
  }

  public keepOriginalOrder = (a, _b) => a.key;

  public formatAddress(): string {
    let address = '';
    if (this.structure.address.numero) address += `${this.structure.address.numero} `;
    if (this.structure.address.street) address += `${this.structure.address.street}, `;
    if (this.structure.address.commune) address += `${this.structure.address.commune}`;
    return address;
  }

  public getAccessLabel(accessModality: AccessModality): string {
    return this.utils.getAccessLabel(accessModality);
  }

  public containsHours(): boolean {
    let contains = false;
    for (const day in this.structure.hours) {
      if (this.structure.hours[day].time.length) contains = true;
    }
    return contains;
  }

  public isFieldValid(field: string): boolean {
    return this.structureForm.get([field]).valid;
  }

  public hasSocialNetworks(): boolean {
    return this.utils.hasSocialNetwork(this.structure);
  }

  public hasWebsite(): boolean {
    return this.utils.hasWebsite(this.structure);
  }

  public getPublicLabel(tagetPublic: PublicCategorie): string {
    return this.utils.getPublicLabel(tagetPublic);
  }

  public containsDigitalHelp(): boolean {
    return !!this.structure.proceduresAccompaniment.length;
  }

  public getDigitalHelpLabel(digitalHelp: Demarches): string {
    return this.utils.getDigitalHelpLabel(digitalHelp);
  }

  // Digital learning
  private async initForm(): Promise<void> {
    this.searchService.getCategoriesTraining().subscribe((referentiels) => {
      referentiels.forEach((referentiel) => {
        if (referentiel.isBaseSkills()) {
          this.baseSkillssReferentiel = referentiel;
        } else if (referentiel.isRigthtsAccess()) {
          this.accessRightsReferentiel = referentiel;
        } else if (referentiel.isDigitalCultureSecurity()) {
          this.digitalCultureSecuritysReferentiel = referentiel;
        } else if (referentiel.isParentingHelp()) {
          this.parentingHelpsReferentiel = referentiel;
        } else if (referentiel.isSocialAndProfessional()) {
          this.socialAndProfessionalsReferentiel = referentiel;
        }
      });
      this.setServiceCategories();
    });
    const index = this.structure.proceduresAccompaniment.indexOf('autres');
    if (index > -1) {
      this.structure.proceduresAccompaniment.splice(index, 1);
    }
  }

  public setServiceCategories(): void {
    this.baseSkills = this.structure.baseSkills.map((skill) =>
      lodash.find(this.baseSkillssReferentiel.modules, { id: skill })
    );
    this.accessRights = this.structure.accessRight.map((rights) =>
      lodash.find(this.accessRightsReferentiel.modules, { id: rights })
    );
    this.parentingHelp = this.structure.parentingHelp.map((help) =>
      lodash.find(this.parentingHelpsReferentiel.modules, { id: help })
    );
    this.socialAndProfessional = this.structure.socialAndProfessional.map((skill) =>
      lodash.find(this.socialAndProfessionalsReferentiel.modules, { id: skill })
    );
    this.digitalCultureSecurity = this.structure.digitalCultureSecurity.map((skill) =>
      lodash.find(this.digitalCultureSecuritysReferentiel.modules, { id: skill })
    );
  }
  public isBaseSkills(): boolean {
    return this.baseSkills && this.baseSkills[0] !== undefined;
  }
  public isAccessRights(): boolean {
    return this.accessRights && this.accessRights[0] !== undefined;
  }
  public isParentingHelp(): boolean {
    return this.parentingHelp && this.parentingHelp[0] !== undefined;
  }
  public isSocialAndProfessional(): boolean {
    return this.socialAndProfessional && this.socialAndProfessional[0] !== undefined;
  }
  public isDigitalSecurity(): boolean {
    return this.digitalCultureSecurity && this.digitalCultureSecurity[0] !== undefined;
  }
  public containsDigitalLearning(): boolean {
    return (
      this.isAccessRights() ||
      this.isBaseSkills() ||
      this.isDigitalSecurity() ||
      this.isParentingHelp() ||
      this.isSocialAndProfessional()
    );
  }
  public toggleBaseSkills(): void {
    this.showBaseSkills = !this.showBaseSkills;
  }
  public toggleAccessRights(): void {
    this.showAccessRights = !this.showAccessRights;
  }
  public toggleParentingHelp(): void {
    this.showParentingHelp = !this.showParentingHelp;
  }
  public toggleSocialAndProfessional(): void {
    this.showSocialAndProfessional = !this.showSocialAndProfessional;
  }
  public toggleDigitalSecurity(): void {
    this.showDigitalSecurity = !this.showDigitalSecurity;
  }

  public getWorkshopPricingLabel(freeWorkshop: FreeWorkshop): string {
    return this.utils.getWorkshopPricingLabel(freeWorkshop);
  }

  public hasWifi(equipmentsAndServices: string[]) {
    return this.utils.hasWifi(equipmentsAndServices);
  }

  public hasEquipments(structure: Structure): boolean {
    return this.utils.hasEquipments(
      structure.equipmentsAndServices,
      structure.nbComputers,
      structure.nbNumericTerminal,
      structure.nbPrinters,
      structure.nbScanners,
      structure.nbTablets
    );
  }

  public filterOnlyEquipments(equipmentsAndServices: string[]): string[] {
    return this.utils.filterOnlyEquipments(equipmentsAndServices);
  }

  public getEquipmentsLabelAndValue(equipment: Equipment, number: number): string {
    return this.utils.getEquipmentsLabelAndValue(equipment, number);
  }

  public filterOtherServices(equipmentsAndServices: string[]) {
    return this.utils.filterOtherServices(equipmentsAndServices);
  }

  public getOtherServicesLabel(otherService: OtherServices) {
    return this.utils.getOtherServicesLabel(otherService);
  }
}
